@@ -253,6 +253,8 @@ tiledb_query_get_buffer_char <- function(bufptr, sizeoffsets=0, sizestring=0) {
 
 #' Submit TileDB Query
 #'
+#' Note that the query object may need to be finalized
+#' via \code{tiledb_query_finalize}.
 #' @param query A TileDB Query object
 #' @return The modified query object, invisibly
 #' @export
@@ -262,6 +264,19 @@ tiledb_query_submit <- function(query) {
   invisible(query)
 }
 
+#' Submit TileDB Query asynchronously without a callback returning immediately
+#'
+#' Note that the query object may need to be finalized
+#' via \code{tiledb_query_finalize}.
+#' @param query A TileDB Query object
+#' @return The modified query object, invisibly
+#' @export
+tiledb_query_submit_async <- function(query) {
+  stopifnot(query_object=is(query, "tiledb_query"))
+  libtiledb_query_submit_async(query@ptr)
+  invisible(query)
+}
+
 #' Finalize TileDB Query
 #'
 #' @param query A TileDB Query object
@@ -333,3 +348,120 @@ tiledb_query_add_range_with_type <- function(query, idx, datatype, lowval, highv
   query@ptr <- libtiledb_query_add_range_with_type(query@ptr, idx, datatype, lowval, highval, stride)
   invisible(query)
 }
+
+
+#' Retrieve the Number of Fragments for Query
+#'
+#' This function is only applicable to \sQuote{WRITE} queries.
+#' @param query A TileDB Query object
+#' @return An integer with the number of fragments for the given query
+#' @export
+tiledb_query_get_fragment_num <- function(query) {
+  stopifnot(query_object=is(query, "tiledb_query"))
+  libtiledb_query_get_fragment_num(query@ptr)
+}
+
+#' Retrieve the URI for a given Query Fragment
+#'
+#' This function is only applicable to \sQuote{WRITE} queries.
+#' @param query A TileDB Query object
+#' @param idx An integer (or numeric) index ranging from zero to the number of fragments minus 1
+#' @return An character value with the fragment URI
+#' @export
+tiledb_query_get_fragment_uri <- function(query, idx) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            idx_numeric=is.numeric(idx))
+  libtiledb_query_get_fragment_uri(query@ptr, idx)
+}
+
+#' Retrieve the timestamp range for a given Query Fragment
+#'
+#' This function is only applicable to \sQuote{WRITE} queries. The time resolution in
+#' TileDB is millseconds since the epoch so an R \code{Datetime} vector is returned.
+#' @param query A TileDB Query object
+#' @param idx An integer (or numeric) index ranging from zero to the number of fragments minus 1
+#' @return A two-element datetime vector with the start and end time of the fragment write.
+#' @export
+tiledb_query_get_fragment_timestamp_range <- function(query, idx) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            idx_numeric=is.numeric(idx))
+  libtiledb_query_get_fragment_uri(query@ptr, idx)
+}
+
+
+#' Retrieve the estimated result size for a query and attribute
+#'
+#' When reading from sparse arrays, one cannot know beforehand how big the
+#' result will be (unless one actually executes the query). This function
+#' offers a way to get the estimated result size for the given attribute.
+#' As TileDB does not actually execute the query, getting the estimated
+#' result is very fast.
+#' @param query A TileDB Query object
+#' @param name A variable with an attribute name
+#' @return An estimate of the query result size
+#' @export
+tiledb_query_get_est_result_size <- function(query, name) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            name_argument=is.character(name))
+  libtiledb_query_get_est_result_size(query@ptr, name)
+}
+
+#' Retrieve the estimated result size for a query and variable-sized attribute
+#'
+#' When reading variable-length attributes from either dense or sparse arrays,
+#' one cannot know beforehand how big the result will be (unless one actually
+#' executes the query). This function offers a way to get the estimated result
+#' size for the given attribute. As TileDB does not actually execute the query,
+#' getting the estimated result is very fast.
+#' @param query A TileDB Query object
+#' @param name A variable with an attribute name
+#' @return An estimate of the query result size
+#' @export
+tiledb_query_get_est_result_size_var <- function(query, name) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            name_argument=is.character(name))
+  libtiledb_query_get_est_result_size_var(query@ptr, name)
+}
+
+#' Retrieve the number of ranges for a query dimension
+#'
+#' @param query A TileDB Query object
+#' @param idx An integer index selecting the dimension
+#' @return An integer with the number of query range for the
+#' given dimensions
+#' @export
+tiledb_query_get_range_num <- function(query, idx) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            idx_argument=is.numeric(idx))
+  libtiledb_query_get_range_num(query@ptr, idx-1)
+}
+
+#' Retrieve the query range for a query dimension and range index
+#'
+#' @param query A TileDB Query object
+#' @param dimidx An integer index selecting the dimension
+#' @param rngidx An integer index selection the given range for the dimension
+#' @return An integer vector with elements start, end and stride for the query
+#' range for the given dimension and range index
+#' @export
+tiledb_query_get_range <- function(query, dimidx, rngidx) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            dimidx_argument=is.numeric(dimidx),
+            rngidx_argument=is.numeric(rngidx))
+  libtiledb_query_get_range(query@ptr, dimidx-1, rngidx-1)
+}
+
+#' Retrieve the query range for a variable-sized query dimension and range index
+#'
+#' @param query A TileDB Query object
+#' @param dimidx An integer index selecting the dimension
+#' @param rngidx An integer index selection the given range for the dimension
+#' @return An string vector with elements start and end for the query
+#' range for the given dimension and range index
+#' @export
+tiledb_query_get_range <- function(query, dimidx, rngidx) {
+  stopifnot(query_object=is(query, "tiledb_query"),
+            dimidx_argument=is.numeric(dimidx),
+            rngidx_argument=is.numeric(rngidx))
+  libtiledb_query_get_range_var(query@ptr, dimidx-1, rngidx-1)
+}
